/* intro binomial data shown in Table 1.1'      */
/* shown below are two analyses                 */
/* 1 - assuming Pct = Y/Nx ~ approx normal      */
/* 2 - as logistic GLM with Y ~ binomial(Nx,pi) */
data ch1_data_set_1;
 input X Nx Y;
datalines;
0 11 0
1 7 0
2 9 2
3 11 2
4 12 2
5 15 5
6 11 7
7 15 12
8 11 10
9 16 16
10 10 9
;
/* run 1 - Y/Nx ~ approx normal      */
proc glimmix data=intro_bin;
pct=y/nx;
 model pct=x/solution; 
      output out=lm_pct_pred pred(ilink)=p_lm;
run;

/* run 2 - Y ~ Binomial(Nx,pi)       */
proc glimmix data=intro_bin;
 model y/nx=x/solution; 
      output out=logit_pred pred(ilink)=p_logit;
run;

/* merge predicted values from run 1 and 2 and plot them 
    to demonstrate how models fit (Figure 1.1 in text      */
proc sort data=lm_pct_pred; by x;
proc sort data=logit_pred; by x;
data to_plot;
 merge logit_pred lin_pct_pred;
  plot='0bserved'; y=pct; output;
  plot='p_GLM'; y=p_pct; output;
  plot='p_Logit'; y=p_logit; output;
 by x;
data compare;
 merge logitpred lin_pct_pred;
 by x;
symbol1 i=none color=black value=star;	
symbol2 i=join line=1 color=red value=diamond;
symbol3 i=join line=2 color=blue value=square; 
legend1 value=(font=swiss h=2  ) label=(f=swiss h=2 'Plot');
proc gplot data=to_plot; 
 plot y*x=plot/legend=legend1;
title 'Figure 1.1'; 
run;
