/* Data Set 11.5                               */
/* Many models are shown here                  */
/* Only Run 4 (ZINB model) appears in textbook */


 /*----------------------------------------------------------*/
 /*---Bio-hazard Data                                     ---*/
 /*--- Overdispersion?      Zero Inflation?               ---*/
 /*----------------------------------------------------------*/
 /*--- Each row = a site                                  ---*/
 /*---First number in row = number of samples at that site---*/
 /*---Each number pair after that = {exp_level tox_count} ---*/

 /*--- Six runs demonstrated                              ---*/
 /*--- 1. Poisson GLMM                                    ---*/
 /*--- 2. NegBin  GLMM                                    ---*/
 /*--- 3. Poisson ZIP                                     ---*/
 /*--- 4. NegBin  ZIP                                     ---*/
 /*--- 5. Poisson Hurdle                                  ---*/
 /*--- 6. NegBin  Hurdle                                  ---*/

data counts;
   input ni @@; 
   site = _n_;
   do sample_id=1 to ni;
     input exposure_level toxin_count @@; output;
   end;
   datalines;
  6  2 0 82 5 33 0 15 2 35 0 79 1 
  9 18 0 64 0 80 2  0 0 58 0  7 0 81 0 22 3 50 0 
  2 34 0 95 0 
 13 28 1 31 0 63 2 14 0 74 0 44 0 75 3 65 0 74 1 84 5 57 0 29 0 41 0 
  9 42 0  8 0 91 0 20 0 23 0 22 0 96 4 83 3 56 0 
  3 64 0 64 1 15 0 
  4  5 0 73 2 50 2 13 1 
  2  0 0 41 0 
  5 83 7 98 1 11 1 28 0 18 0 
  7 91 0 25 1 51 4 20 0 61 1 34 0 33 2 
 14 60 0 87 0 94 0 29 0 41 0 78 0 50 0 37 0 15 0 39 0 22 0 82 0 93 0 3 0 
 12 68 0 26 1 19 0 60 1 93 3 65 0 16 0 79 0 14 0  3 1 90 0 28 3
  8 34 1 44 0 62 3 21 0  7 0 17 1  0 2 49 0 
 13 11 0 27 2 16 1 12 3 52 1 55 0  2 6 89 5 31 5 28 3 51 5 54 13 64 0 
  9  3 0 36 0 57 0 77 0 41 0 39 0 55 0 57 0 88 1 
  7  2 0 80 0 41 1 20 0  2 0 27 0 40 0 
 13 59 0 96 2 47 1 64 0 18 0 30 0 37 0 36 1 69 0 78 1 47 1 86 0 88 0 
 12 84 6 60 1 33 1 92 0 38 4  6 0 43 3 13 2 18 0 51 0 50 4 68 0
 ;
 proc print; run;

/* Run 1: original model assuming Poisson counts */
/* chi-sq/df=1.46, AICC=418.97                   */
proc glimmix data=counts method=quad;
   class site;
   model toxin_count = exposure_level / dist=poisson s;
   random int / subject=site;
   ods select FitStatistics CondFitStatistics CovParms ParameterEstimates;
run;

/* Run 2: GLMM assuming Negative Binomial counts */
/* chi-sq/df=0.67, AICC=377.02                   */
proc glimmix data=counts method=quad;
   class site;
   model toxin_count = exposure_level / dist=negbin s;
   random int / subject=site;
   estimate 'yhat at x=10' intercept 1 exposure_level 10/ilink;
   *ods select FitStatistics CondFitStatistics CovParms ParameterEstimates;
run;

/* FYI only - ZIP model using Proc GENMOD			    					 */
/* Note: Bogus because GenMod doesn't know how to deal with random effect    */
/* but it does estimate inflation probability: est inf_prob=0.5675           */
/*--- but also note it will only estimate inflation prob if you leave site   */
/*    out of the model                                                    ---*/
proc genmod data=counts;
 class site;
 model toxin_count = /*site*/ exposure_level / dist=zip;
 zeromodel /link=logit;
 estimate ' ' @zero int 1;
run;

/* Run 3: Proc NLMIXED: random site effect, ZIP model    */
/* inflation prob estimate = 0.389, AICC=384.5           */
proc nlmixed data=counts;
   parameters beta0=0 beta1=0
              a0=0 s2i=1; 
  	x=exposure_level;
  	y=toxin_count;
  /* linear predictor for the inflation probability      */
   LinPr_Infl = a0;
  /* infprob = inflation probability for zeros           */
  /*         = logistic transform of the linear predictor*/
   InfProb  = 1/(1+exp(-LinPr_Infl));
  /* Poisson mean */
   lambda   = exp(beta0+b0 + beta1*x);
  /* Build the ZIP log likelihood */
   if y=0 then 
        ll = log(infprob + (1-infprob)*exp(-lambda));
   else ll = log((1-infprob)) + y*log(lambda) - lgamma(y+1) - lambda;
   model y ~ general(ll);
   random b0  ~ normal([0],[s2i]) subject=site;
   estimate "inflation probability" infprob; 
run;

/* Run 4: zero-inflated negative binomial model          */
/* inflation prob estimate essentially 0, AICC=379.2     */
proc nlmixed data=counts;
 parms a0=0 scale=0.5 beta0=0 beta1=0 sd2_0=1; 
  	x=exposure_level;
  	y=toxin_count;
 /* lin predictor for zero-inflation probability         */
 LinPr_Infl = a0;
 /* InfProb=prob(zero) due to zero-inflation process     */
 /* note: it is the logistic i-link of lp_pi             */
 InfProb  = 1/(1+exp(-LinPr_Infl));
 /* neg bin mean (lamda)                                 */
 eta=beta0+b0+beta1*x;
 lamda=exp(eta);
 /* build "y+n+1 chose y" part of neg bin logL           */
 /* note SAS has no combinatoric "choose" function,
      so you use the gamma function                      */
 /* LGamma is the log(gamma) function                    */
 nn=1/scale;
 /* note "pp=nn/(l_hat+nn)" is the prob in 
      neg bin pdf you see in Math Stat class             */
  log_choose=lgamma(y+nn)-lgamma(y+1)-lgamma(nn);
 /* define logL for ZINB                                 */
 if y=0 then ll=log(InfProb +(1-InfProb))-log(1+lamda*scale)/scale;
  else ll=log(1-InfProb)+y*log(lamda*scale/(1+lamda*scale))-log(1+lamda*scale)/scale+log_choose;
 /* model & random statement                             */
model y~general(ll);
 random b0 ~ normal([0],[sd2_0*sd2_0]) subject=site;
/* estimate various things of interest                   */
 estimate 'var_b0' sd2_0*sd2_0;
 estimate 'yhat at x=0' exp(beta0);
 estimate 'yhat at x=10' exp(beta0+beta1*10);
 estimate 'yhat at x=20' exp(beta0+beta1*20);
 estimate 'inflation prob' InfProb;
run;


/* Run 5: Proc NLMIXED: Hurdle-Poisson model            */
/* inflation prob estimate = 0.622, AICC=399.2          */
proc nlmixed data=counts;
   parameters beta0=0 beta1=0
              a0=0 s2i=1; 
  	x=exposure_level;
  	y=toxin_count;
  /* linear predictor for the inflation probability      */
   LinPr_Infl = a0;
  /* infprob = inflation probability for zeros           */
  /*  assume Logit Hrdle zero-inflation                  */
   InfProb  = 1/(1+exp(-LinPr_Infl));
  /* Poisson mean */
   lambda   = exp(beta0+b0 + beta1*x);
  /* Build the Hurdle log likelihood */
   if y=0 then 
        ll = log(infprob);
   else ll = log((1-infprob)) + y*log(lambda)-lgamma(y+1)-lambda - log(1-exp(-lambda));
   model y ~ general(ll);
   random b0  ~ normal([0],[s2i]) subject=site;
   estimate "inflation probability" infprob;
run;

/* Run 6: Proc NLMIXED: Hurdle-NegBin model             */
/* inflation prob estimate = 0.622, AICC=553.1          */
proc nlmixed data=counts;
   parameters scale=0.5 beta0=0 beta1=0
              a0=0 s2i=1; 
  	x=exposure_level;
  	y=toxin_count;
  /* linear predictor for the inflation probability      */
   LinPr_Infl = a0;
  /* infprob = inflation probability for zeros           */
  /*  assume Logit Hrdle zero-inflation                  */
   InfProb  = 1/(1+exp(-LinPr_Infl));
  /* NegBin mean */
   lamda   = exp(beta0+b0 + beta1*x);
  /* Build the Hurdle log likelihood */
  nn=1/scale;
  log_choose=lgamma(y+nn)-lgamma(y+1)-lgamma(nn);
   if y=0 then 
        ll = log(infprob);
   else ll = log((1-infprob)) + 
            +y*log(lamda*scale/(1+lamda*scale))-log(1+lamda*scale)/scale+log_choose
            -(log(1-(1/(1+lamda*scale))**(1/scale)));
   model y ~ general(ll);
   random b0  ~ normal([0],[s2i]) subject=site;
   estimate "inflation probability" infprob;
run;

