/* Data Set 11.3 - Randomized Complete Block Design with Counts  */
data RCBD_counts;
 input blk trt count;
 y=count;
datalines;
1 1 4
1 2 5
1 3 2
2 1 4
2 2 1
2 3 3
3 1 6
3 2 4
3 3 3
4 1 11
4 2 28
4 3 121
5 1 13
5 2 61
5 3 45
6 1 20
6 2 1
6 3 14
7 1 20
7 2 2
7 3 6
8 1 2
8 2 14
8 3 46
9 1 0
9 2 13
9 3 15
10 1 0
10 2 4
10 3 2
;
/* Run 1 - Poisson model in Laplace  */
/* Pearson conditional chi-square is appropriate overdispersion diagnostic  */
proc glimmix method=laplace data=RCBD_counts;
 class blk trt;
  model y=trt/dist=poisson;
  random intercept  / subject=blk;
  lsmeans trt/ilink;
  title 'Laplace - "standard" Poisson model';
  ods listing select fitstatistics condfitstatistics covparms tests3 lsmeans;
run;

/* Run 2 - Poisson model with "classical" Overdispersion correction  */
/* scale parameter fix - note PL method required here - no Laplace   */
proc glimmix /*method=laplace*/ data=RCBD_counts;
 class blk trt;
  model y=trt/dist=poisson s;
  random intercept  / subject=blk;
  random _residual_;
  lsmeans trt/ilink;
  title ' Poisson model + OD scale parameter';
  *ods listing select fitstatistics covparms tests3 lsmeans;
run;

/* Run 3 - Poisson model in Laplace  */
/*  fix via adding blk*trt effect to random statement  */
proc glimmix method=laplace data=RCBD_counts;
 class blk trt;
  model y=trt/dist=poisson;
  random intercept trt / subject=blk;
  lsmeans trt/ilink;
  title ' Poisson model + blk*trt effect';
  ods listing select fitstatistics condfitstatistics covparms tests3 lsmeans;
run;

/* Run 4 - Negative Binomial model in Laplace  */
/*  delete blk*trt effect - need it to estimate NegBin scale parameter  */
proc glimmix method=laplace data=RCBD_counts;
 class blk trt;
  model y=trt/dist=negbin;
  random intercept / subject=blk;
  lsmeans trt/ilink;
  covtest 'scale=0' . 0;
  title ' Negative Binomial model ';
  ods listing select fitstatistics condfitstatistics covparms tests3 lsmeans covtests;
run;

/* Run 5 - Generalized Poisson model in Laplace  */
/*  delete blk*trt effect - need it to estimate NegBin scale parameter  */
proc glimmix method=laplace /*method=quad*/ data=RCBD_counts;
 class blk trt;
  model y=trt/ link=log s;
   random int / subject=blk;
   xi = (1 - 1/exp(_phi_));
   _variance_ = _mu_ / (1-xi)/(1-xi);
   if (_mu_=.) or (_linp_ = .) then _logl_ = .;
   else do;
      mustar = _mu_ - xi*(_mu_ - y);
      if (mustar < 1E-12) or (_mu_*(1-xi) < 1e-12) then
         _logl_ = -1E20;
      else do;
         _logl_ = log(_mu_*(1-xi)) + (y-1)*log(mustar) - 
                   mustar - lgamma(y+1);
      end;
   end;
  lsmeans trt/ilink;
  covtest 'scale=0' . 0;
  title ' Generalized Poisson model ';
  *ods listing select fitstatistics condfitstatistics covparms tests3 lsmeans covtests;
run;

