/* Example Illustrating Creation of Covariance Plots      */
/* Repeated Measures Data Used in Ch 6 Motivating Example */
/*   Data Set 6.2                                         */
/* Data First presented in Multivariate form              */
/*   (suitable for Proc GLM)                              */
data multivar;
 input treatment subj_id time1-time6;
datalines;
1 1 9.326 7.936 6.175 6.891 7.468 7.018
1 2 9.086 8.013 6.247 6.462 5.355 6.236
1 3 10.584 10.348 8.622 8.429 7.252 7.825
1 4 11.866 10.975 10.391 9.143 8.201 7.861
1 5 8.624 8.301 7.016 6.012 5.753 4.328
1 6 10.703 8.907 8.624 7.54 7.262 6.9
2 1 9.471 8.05 7.073 6.65 6.942 5.949
2 2 10.356 8.661 6.789 5.682 6.026 5.879
2 3 10.379 8.068 6.443 6.247 5.798 6.56
2 4 8.062 7.143 5.179 2.281 2.888 3.633
2 5 8.843 7.51 6.779 4.497 3.918 5.723
2 6 11.198 8.301 6.854 5.331 5.532 5.756
;
/* convert to univariate form (suitable for Proc MIXED, GLIMMIX */
data univar;
 set multivar;
 time=1; y=time1; output;
 time=2; y=time2; output;
 time=3; y=time3; output;
 time=4; y=time4; output;
 time=5; y=time5; output;
 time=6; y=time6; output;

/* output raw unstructured covariance estimates */
/* no REML estimation                           */
/* Just Sums of squares and cross-products      */
 proc mixed data=univar;
 class treatment time subj_id;
  model y=treatment|time; 
 repeated / type=un sscp subject=subj_id(treatment) rcorr;
ods output covparms=cov;
ods output rcorr=corr;
run;

/* create data set for covariance plots & adjusted covariance & correlation plots  */
data times;
 do time1=1 to 6;
  do time2=1 to time1;
   dist=time1-time2;
   output;
  end;
 end;
data covplot;
 merge times cov corr;
proc print;
run;

/* define labels for plots */
title ' ';
*axis1 value=(font=swiss2 h=2) label=(angle=90 f=swiss h=2 'Correlation adj for Between Subj Var');
axis1 value=(font=swiss2 h=2) label=(angle=90 f=swiss h=2 'Covariance of Within Subject Effects')
  order=0 to 2 by 0.2;
axis2 value=(font=swiss h=2 )label=(f=swiss h=2 'Distance');
legend1 value=(font=swiss h=2  ) label=(f=swiss h=2 'From Time');
symbol1 color=black interpol=join
         line=1 value=square; 
symbol2 color=black interpol=join
       line=2 value=circle; 
symbol3 color=black interpol=join
        line=20 value=triangle;
symbol4 color=black interpol=join
         line=3 value=plus; 
symbol5 color=black interpol=join
       line=4 value=star; 
symbol6 color=black interpol=join
        line=5 value=dot; 
symbol7 color=black interpol=join
       line=6 value=_; 
symbol8 color=black interpol=join
        line=10 value==;

/* plot, using Proc GPLOT */
proc gplot data=covplot;
 plot estimate*dist=time2/vaxis=axis1 haxis=axis2 legend=legend1; 
 run;

/* fit AR(1) model */
 proc glimmix data=univar;
  class treatment time subj_id;
  model y=treatment|time;
  random intercept / subject=subj_id(treatment); 
  random _residual_ / type=ar(1) subject=subj_id(treatment);
 run;

/* create idealized plot from AR(1) estimates    */
/* to visualize what AR(1) covariance looks like */ 
 data plotar1;
  do distance=0 to 5;
  b_var=1.0427;
  rho=0.4109;
  w_var=0.4596;
  pure_ar1=w_var*(rho**distance);
  cs=b_var;
  ar1_plus_subj=pure_ar1+b_var;
  output;
  end;
symbol i=join;
 proc gplot data=plotar1;
  plot (cs pure_ar1 ar1_plus_subj)*distance / overlay vaxis=axis1 haxis=axis2 legend=legend1; ;
 run;
