/* Chapter 16, Example 16.5               */
/* different designs for multi-site study */

/* design 1: BIB */
data bib;
input loc @@;
n_subj=100;
 do g=1 to 2;
  input trt @@;
   do r=1 to 1; /* R = # multiples of locations used to increase power */
   location=(r-1)*6+loc;
   pi=0.15*(trt=0)+0.25*(trt=1)+0.30*(trt=2)+0.35*(trt=3);
   mu=n_subj*pi;
   output;
  end;
end;
datalines;
1  0 1
2  0 2
3  0 3
4  1 2
5  1 3
6  2 3
;
proc sort data=bib;
 by location;
proc print data=bib;
run; 
proc glimmix data=bib initglm;
 class location trt;
 model mu/n_subj=trt;
 random intercept trt / subject=location;
 parms (0.10)(0.05)/hold=1,2;
/*random _residual_;
 parms (0.5)(1)(0.99)/hold=1,2,3;*/
 contrast 'c vs e1' trt 1 -1 0 0;
 contrast 'c vs e2' trt 1 0 -1 0;
 contrast 'c vs e3' trt 1 0 0 -1;
 contrast 'e1 v e3' trt 0 1 0 -1;
 ods output contrasts=f_tests_b;
run;
proc print data=f_tests_b;
run;
data power_bib;
 set f_tests_b;
 alpha=0.05;
 ncp=numdf*fvalue;
 f_crit=finv(1-alpha,numdf,dendf,0);
 power=1-probF(f_crit,numdf,dendf,ncp);

proc print data=power_bib;
run;

/* design 2 - CVT */
data cvt;
input loc @@;
 do g=1 to 2;
  input trt @@;
 do r=1 to 1;
   location=(r-1)*6+loc;
   n_subj=50*(trt=0)+150*(trt>0);
   
   mu=n_subj*(0.15*(trt=0)+0.25*(trt=1)+0.30*(trt=2)+0.35*(trt=3));
   output;
  end;
end;
datalines;
1  0 1
2  0 2
3  0 3
4  0 1
5  0 2
6  0 3
;
proc sort data=cvt;
 by location;
proc print data=cvt;
run; 
proc glimmix data=cvt initglm;
 class location trt;
 model mu/n_subj=trt;
 random intercept trt / subject=location;
 parms (0.10)(0.05)/hold=1,2;
/*random _residual_;
 parms (0.10)(0.05)(0.99)/hold=1,2,3;*/
 contrast 'c vs e1' trt 1 -1 0 0;
 contrast 'c vs e2' trt 1 0 -1 0;
 contrast 'c vs e3' trt 1 0 0 -1;
 contrast 'e1 v e3' trt 0 1 0 -1;
 ods output contrasts=f_tests_c;
run;
proc print data=f_tests_c;
run;
data power_cvt;
 set f_tests_c;
 alpha=0.05;
 ncp=numdf*fvalue;
 f_crit=finv(1-alpha,numdf,dendf,0);
 power_cvt=1-probF(f_crit,numdf,dendf,ncp);
proc print data=power_cvt;
run;

/* design 3 - RCB  */
data rcb;
do u=1 to 4;
input trt @@;
   n_subj=100;
 do block=1 to 3;
   mu=n_subj*(0.15*(trt=0)+0.25*(trt=1)+0.30*(trt=2)+0.35*(trt=3));
   output;
  end;
  end;
datalines;
0 1 2 3
;
proc sort data=rcb;
 by block;
proc print data=rcb;
run; 
proc glimmix data=rcb initglm;
 class block trt;
 model mu/n_subj=trt;
 random intercept trt / subject=block;
 parms (0.012)(0.094)/hold=1,2;
/*random _residual_;
 parms (0.012)(0.094)(0.99)/hold=1,2,3;*/
 contrast 'c vs e1' trt 1 -1 0 0;
 contrast 'c vs e2' trt 1 0 -1 0;
 contrast 'c vs e3' trt 1 0 0 -1;
 contrast 'e1 v e3' trt 0 1 0 -1;
 ods output contrasts=f_tests_r;
run;
proc print data=f_tests_r;
run;
data power_rcb;
 set f_tests_r;
 alpha=0.05;
 ncp=numdf*fvalue;
 f_crit=finv(1-alpha,numdf,dendf,0);
 power_rcb=1-probF(f_crit,numdf,dendf,ncp);
proc print data=power_rcb;
run;
