/* Section 16.3 */
/* comparison of altenative designs for 10x3 layout with 6 trts, Gaussian Data */
/* version shown here with natural block variance = 5, e.u. variance = 1       */

/* Balanced Incomplete Block (BIB) */

/* Step 1: create exemplary data set  */
data bib;
 input block @@;
  do plot=1 to 3;
   input trt @@; 
    mu=20-1.25*((trt=1)+2*(trt=2));  /* MU is value to response diffs to be assesed in power analysis  */
   output;
  end;
datalines;
 1  0 1 2
 2  0 1 3
 3  0 2 4
 4  0 3 5
 5  0 4 5
 6  1 2 5
 7  1 3 4
 8  1 4 5
 9  2 3 4
10  2 3 5
;
/* Step 2: GLIMMIX step */
proc glimmix data=bib;
 class block trt;
 model mu=trt;
 random intercept /subject=block;
 parms (5)(1) / hold=1,2;             
 lsmeans trt / diff;                    
 contrast 'Set x 0 v 2' trt 1 0 -1 -1 0 1; 
 contrast 'Trt_0 v Trt_1' trt 1 -1 0;
 contrast 'Trt_0 v Trt_2' trt 1 0 -1 0;
 contrast 'Trt_1 v Trt_4' trt 0 1 0 0 -1 0;
 contrast 'Trt_2 v Trt_5' trt 0 0 1 0 0 -1;
 ods output contrasts=c;
run;
/* Sept 3; Power step  */
data power;
 set c;
 alpha=0.05;
 ncparm=numdf*fvalue;
 fcrit=finv(1-alpha,numdf,dendf,0);
 power=1-probf(fcrit,numdf,dendf,ncparm);
/* step 4: print results of power calculation  */
proc print data=power; 
 var Label NumDF DenDF NCParm FCrit Power;
 title 'Balanced Incomplete Block';
run; 

/* Control vs Treated (CVT) */

/* Step 1: create exemplary data set  */
data cvt;
 input block @@;
  do plot=1 to 3;
   input trt @@; 
    mu=20-1.25*((trt=1)+2*(trt=2));  /* MU is value to response diffs to be assesed in power analysis  */
   output;
  end;
datalines;
 1  0 1 2
 2  0 1 3
 3  0 1 4
 4  0 1 5
 5  0 2 3
 6  0 2 4
 7  0 2 5
 8  0 3 4
 9  0 3 5
10  0 4 5
;
/* Step 2: GLIMMIX step */
proc glimmix data=cvt;
 class block trt;
 model mu=trt;
 random intercept /subject=block;
 parms (5)(1) / hold=1,2;             
 lsmeans trt / diff;                    
 contrast 'Set x 0 v 2' trt 1 0 -1 -1 0 1; 
 contrast 'Trt_0 v Trt_1' trt 1 -1 0;
 contrast 'Trt_0 v Trt_2' trt 1 0 -1 0;
 contrast 'Trt_1 v Trt_4' trt 0 1 0 0 -1 0;
 contrast 'Trt_2 v Trt_5' trt 0 0 1 0 0 -1;
 ods output contrasts=c;
run;
/* Sept 3; Power step  */
data power_cvt;
 set c;
 alpha=0.05;
 ncparm=numdf*fvalue;
 fcrit=finv(1-alpha,numdf,dendf,0);
 Power_CVT=1-probf(fcrit,numdf,dendf,ncparm);
/* step 4: print results of power calculation  */
proc print data=power_cvt; 
 var Label NumDF DenDF NCParm FCrit Power_CVT;
 title 'Balanced Incomplete Block';
run; 

/* Disconnected / Nested Factorial / Split-Plot (DNC) */

/* Step 1: create exemplary data set  */
data dnf;
 input block @@;
  do plot=1 to 3;
   input trt @@; 
    mu=20-1.25*((trt=1)+2*(trt=2));  /* MU is value to response diffs to be assesed in power analysis  */
   output;
  end;
datalines;
 1  0 1 2
 2  0 1 2
 3  0 1 2
 4  0 1 2
 5  0 1 2
 6  3 4 5
 7  3 4 5
 8  3 4 5
 9  3 4 5
10  3 4 5
;
/* Step 2: GLIMMIX step */
proc glimmix data=dnf;
 class block trt;
 model mu=trt;
 random intercept /subject=block;
 parms (5)(1) / hold=1,2;             
 lsmeans trt / diff;                    
 contrast 'Set x 0 v 2' trt 1 0 -1 -1 0 1; 
 contrast 'Trt_0 v Trt_1' trt 1 -1 0;
 contrast 'Trt_0 v Trt_2' trt 1 0 -1 0;
 contrast 'Trt_1 v Trt_4' trt 0 1 0 0 -1 0;
 contrast 'Trt_2 v Trt_5' trt 0 0 1 0 0 -1;
 ods output contrasts=c;
run;
/* Sept 3; Power step  */
data power_dnf;
 set c;
 alpha=0.05;
 ncparm=numdf*fvalue;
 fcrit=finv(1-alpha,numdf,dendf,0);
 Power_DNF=1-probf(fcrit,numdf,dendf,ncparm);
/* step 4: print results of power calculation  */
proc print data=power_dnf; 
 var Label NumDF DenDF NCParm FCrit Power_DNF;
 title 'Balanced Incomplete Block';
run; 


/* Randomized Complete Block  */

/* Step 1: create exemplary data set  */
data rcb;
 do rep=1 to 5;
  do trt=0 to 5;
   mu=20-1.25*((trt=1)+2*(trt=2));  /* MU is value to response diffs to be assesed in power analysis  */
   output;
  end;
 end;

/* Step 2: GLIMMIX step */
proc glimmix data=rcb noprofile;
 class rep trt;
 model mu=trt;
 random intercept /subject=rep;
 parms (2)(4) / hold=1,2;          
 contrast 'Set x 0 v 2' trt 1 0 -1 -1 0 1; 
 contrast 'Trt_0 v Trt_1' trt 1 -1 0;
 contrast 'Trt_0 v Trt_2' trt 1 0 -1 0;
 contrast 'Trt_1 v Trt_4' trt 0 1 0 0 -1 0;
 contrast 'Trt_2 v Trt_5' trt 0 0 1 0 0 -1;
 ods output contrasts=c;
run;
/* Sept 3; Power step  */
data power_rcb1;
 set c;
 alpha=0.05;
 ncparm=numdf*fvalue;
 fcrit=finv(1-alpha,numdf,dendf,0);
 Power_RCB=1-probf(fcrit,numdf,dendf,ncparm);
/* step 4: print results of power calculation  */
proc print data=power_rcb1; 
 var Label NumDF DenDF NCParm FCrit Power_RCB;
 title 'Randomized Complete Block';
run; 
