/* SAS for MM, 2nd ed, Chapter 2, Cochran & Cox PBIB example    */
/* various standard error options                               */
/*  F- and std error consequences shown in Chapter 5, Section E */

options ls=78;

data cc456;
 input bloc trt y @@;
datalines;
 1 15 24  1  9 25  1  1 26  1 13 20
 2  5 27  2  7 28  2  8 24  2  1 27
 3 10 26  3  1 28  3 14 24  3  2 24
 4 15 34  4 11 31  4  2 21  4  3 23
 5  6 41  5 15 33  5  4 33  5  7 29
 6 12 34  6  4 32  6  3 28  6  1 30
 7 12 32  7 14 25  7 15 24  7  8 26
 8  6 23  8  3 23  8 14 24  8  5 27
 9  5 28  9  4 28  9  2 26  9 13 25
10 10 25 10 12 27 10 13 28 10  6 26
11  9 26 11  7 26 11 10 23 11  3 24
12  8 27 12  6 27 12  2 25 12  9 26
13  5 30 13  9 36 13 11 32 13 12 32
14  7 30 14 13 28 14 14 24 14 11 25
15 10 24 15  4 25 15  8 32 15 11 31
;
/* default REML analysis  */
proc glimmix data=cc456;
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods listing exclude lsmeans diffs;
  ods output diffs=remlmdldiff;
 run;
proc means data=remlmdldiff;
 var stderr;
run;
/* REML with Kenward-Roger correction */
proc glimmix data=cc456;
 class trt bloc;
  model y=trt/ddfm=kr;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods listing exclude lsmeans diffs;
  ods output diffs=remlmdldiffkr;
 run;
proc means data=remlmdldiffkr;
 var stderr;
run;
/* maximum likelihood, uncorrected model-based estimates */
proc glimmix data=cc456 method=mspl;
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods listing exclude lsmeans diffs;
  ods output diffs=mlmdldiff;
 run;
proc means data=mlmdldiff;
 var stderr;
run;
/* maximum likelihood, Kenward-Roger corrected model-based estimates */
proc glimmix data=cc456 method=mspl;
 class trt bloc;
  model y=trt/ddfm=kr;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods listing exclude lsmeans diffs;
  ods output diffs=mlmdldiffkr;
 run;
proc means data=mlmdldiffkr;
 var stderr;
run;
/* REML, classical empirical - a.k.a. "sandwich" - estimates */
proc glimmix data=cc456 empirical;
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods listing exclude lsmeans diffs;
  ods output diffs=remlsndwchdiff;
 run;
proc means data=remlsndwchdiff;
 var stderr;
run;
/* ML with uncorrected sandwich (classical empirical) estimates */
proc glimmix data=cc456 empirical method=mspl;
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods listing exclude lsmeans diffs;
  ods output diffs=mlmbndiff;
 run;
proc means data=mlmbndiff;
 var stderr;
run;
/* REML with Morel-coreected sandwich estimates */
proc glimmix data=cc456 empirical=mbn; 
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods output diffs=remlmbndiff;
 run;
proc means data=remlmbndiff;
 var stderr;
run;
/* ML with Morel corrected sandwich estimates */
proc glimmix data=cc456 empirical=mbn method=mspl;
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods listing exclude lsmeans diffs;
  ods output diffs=mlmbndiff;
 run;
proc means data=mlmbndiff;
 var stderr;
run;
/* REML with mbn tweaked to get se(diff) approximately equal to model-based */
proc glimmix data=cc456 empirical=mbn(d=7.5 r=0.1); 
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods output diffs=remlmbndiff;
 run;
proc means data=remlmbndiff;
 var stderr;
run;

/* not shown in text - ML via Laplace method with uncorrected sandwich estimates */
proc glimmix data=cc456 empirical method=laplace;
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods output diffs=mlsndwchdiff;
  ods listing exclude lsmeans diffs;
 run;
proc means data=mlsndwchdiff;
 var stderr;
run;
/* also not shown in text - Laplace, Morel-corrected sandwich estimates */
proc glimmix data=cc456 empirical=mbn method=laplace;
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods output diffs=mlmbndiff;
 run;
proc means data=mlmbndiff;
 var stderr;
run;
/* also not shown - ML via quadrature, Morel-corrected sandwich */
proc glimmix data=cc456 empirical=mbn method=quad(qpoints=7);
 class trt bloc;
  model y=trt;
  random intercept / subject=bloc;
  lsmeans trt / diff;
  ods output diffs=mlmbndiff;
 run;
proc means data=mlmbndiff;
 var stderr;
run;
