/* simulation of multi-lcoation logit example          */
 
/* simulation of 2-trt paired logit example          */
data sim;
 seed1=667519640;
 seed2=195686112;
 seed3=061872010;
 prob0=0.9;
 prob1=0.9;
 alpha0=log(prob0/(1-prob0));
 alpha1=log(prob1/(1-prob1));
 /* set number of Technicians  */
 N_Tech=10;
 /* set variance for Techniciann random effect */
 Tech_variance=1.2;
 VT_Variance=0.5;
 N_Exp=5000;
 
 /* generate data for N_Exp experiments  */
do expt=1 to N_Exp;
 do Tech=1 to N_Tech;
  T_i=sqrt(Tech_variance)*rannor(seed1);
  do Variety='A','B';
  VT_ij=sqrt(VT_variance)*rannor(seed2);   /* L_i is random Location effect                          */
  eta=(variety='A')*alpha0+(variety='B')*alpha1+T_i+VT_ij; /* eta is linear predictor on logit scale                 */
  p_i=1/(1+exp(-eta));                   /* p_i is true conditional probability for i_th Location  */
  n_i=100;                                /* n_i is number of Patients at i_th Location             */
   * n_i=10;
  y_i=ranbin(seed3,n_i,p_i);              /* y_i is observed number of favorable outcomes           */
  prop=y_i/n_i;
  output;
 end;
end;
end;
;
/* generate observed F-values to test equality of trt effects */
/* here: 5000 simulated experiments                           */ 
ods results off;
ods listing exclude all;
proc glimmix data=sim;
 *where expt<5;
 by expt;
 class Tech Variety;
 model y_i/n_i=variety; * /ddfm=kr;
 random intercept variety/ subject=Tech;
 lsmeans Variety / diff ilink;
 nloptions  maxiter=500;
 ods output tests3=F_obs;
run;

ods listing select all;
ods results on;
/* create data set with observed percent of F-values */
/* for clarity in plot, done in increments of 0.02   */
data F_values;
 set F_obs;
 F_value=round(Fvalue,0.02);
proc freq data=F_values;
 table F_value / out=empirical_f;
run;
/* generate actual pdf of F under H_0  */
data F_dist;
 do x=0.02 to 10 by 0.02;
  *f_X2=pdf('chisquare',x,1,0);
  f_F=pdf('F',x,1,9,0);
  F_Value=round(x,0.02);
  output;
 end;
/* optional - plot the F pdf */
 symbol i=join;
proc gplot data=F_dist;
 plot f_F*x/area=1;
run;

/* merge the empirical and true pdf  */
proc sort data=F_dist;
 by F_value;
proc sort data=empirical_f;
 by F_value;
data graph_F_obs_v_empirical;
 merge F_dist empirical_f;
  by F_value;
  if percent=. then percent=0;
  pct=(1/2)*percent;
  F_Value=round(F_value,0.02);
run;

/* plot the pdf (f_F) vs the observed percent by F_value                    */
/* note - in this plot "percent" is roughly double the pdf values           */
/* new variable PCT created so actual and empirical pdf on comparable scale */
symbol i=none value=star;
proc gplot data=graph_F_obs_v_empirical;
 plot f_F*percent;
run;

/* finally, plot observed versus actual pdf               */
/* statements below are for presentability of plot        */
/* this is the actual SAS code that generated Figurte 5.1 */
 
/* ---------- Setting horizonal axis titles and labels options. ---------- */
/* Horizontal */     axis1   label=(height=2 font='Duplex' "F-value")
/* Xaxis      */             order=0 to 10 by 1
/* Xaxis      */             value=(height=1 font='Duplex')
/* Xaxis      */             offset=(2,1)  /* left,right */
/* Xaxis      */             width=1.25            /* Thicknes of axis/frame line */
/* Xaxis      */             major=(h=-.35)        /* Major tick mark */
/* Xaxis      */             minor=(h=-.35 n=1);    /* Minor tick mark */
 
/* ---------- Setting verical axis titles and labels options. ---------- */
/* Vertical   */     axis2   label=(height=2 angle=90 font='Duplex'  "pdf/pct")
/*      Yaxis */             order=0 to 5 by 1
/*      Yaxis */             value=(height=1 font='Duplex')
/*      Yaxis */             offset=(1 ,3) /* bottom,top */
/*      Yaxis */             width=1.25            /* Thicknes of axis/frame line */
/*      Yaxis */             major=(h=-.5)         /* Major tick mark */
/*      Yaxis */             minor=(h=-.35 n=1);   /* Minor tick mark */
 
/* ---------- Defining line type, color, and width options. ---------- */

goptions=reset;
symbol1 i=join value=none color=black line=1 width=2;
symbol2 i=needle value=none color=gray;
proc gplot data=graph_F_obs_v_empirical;
 plot ( f_F pct)*F_value/overlay haxis=axis1 vaxis=axis2 /*areas=2*/;
 title 'Observed Distribution of Approximate F (gray) versus Actual F pdf (black)';
run;

