/* Example 5.C.1                              */
/* logistic multiple regression data          */
/* these data are simulated via the following */
data a;
seed=23012012;
 do x=0,1,2,4,8,12,16;
  y=-0.55+0.45*x-0.025*x*x+0.00001*x*x*x;
  do eu=1 to 2;
   eta=y+0.5*rannor(seed);
   p=1/(1+exp(-eta));
   n=round(40+10*ranuni(seed));
   f=ranbin(seed,n,p);
   output;
  end;
 end;
 
/* create figure 5.2 */
goptions reset=all gsfmode=replace gsfname=grafout;
goptions ftext='duplex';
 
/* ---------- Setting horizonal axis titles and labels options. ---------- */
/* Horizontal */     axis1   label=(height=2 font='duplex' "X")
/* Xaxis      */             order=0 to 20 by 2
/* Xaxis      */             value=(height=1 font='duplex')
/* Xaxis      */             /*offset=(2,1)*/  /* left,right */
/* Xaxis      */             /*width=1.25 */           /* Thicknes of axis/frame line */
/* Xaxis      */             /*major=(h=-.35)*/        /* Major tick mark */
/* Xaxis      */             /*minor=(h=-.35 n=1)*/;    /* Minor tick mark */
 
/* ---------- Setting verical axis titles and labels options. ---------- */
/* Vertical   */     axis2   label=(height=2 angle=90 font='duplex'  "Logit")
/*      Yaxis */             order=-1 to 3 by .5
/*      Yaxis */             value=(height=1 font='duplex')
/*      Yaxis */             /*offset=(1 ,3)*/ /* bottom,top */
/*      Yaxis */             /*width=1.25*/            /* Thicknes of axis/frame line */
/*      Yaxis */             /*major=(h=-.5)         /* Major tick mark */
/*      Yaxis */             /*minor=(h=-.35 n=1)*/;   /* Minor tick mark */
 
/* ---------- Defining line type, color, and width options. ---------- */
symbol1 value=star i=rq line=20 c=black /*width=1*/;
 proc print data=a;
 var x f n;
 run;
 data plot;
 set a;
 prop=f/n;
 logit=log(prop/(1-prop));
 proc gplot data=plot;
  plot logit*x / Haxis=axis1 Vaxis=axis2;
  title 'plot of logit by X';
 run;
ods rtf close;
quit;
run;

/* SAS PROC GENMOD computes deviance statistics referenced in the text */
/* type 1 are sequential */
/* type 3 are partical   */
proc genmod data=a;
 model f/n=x/type1wald type1;
run;
proc genmod data=a;
 model f/n=x|x/type1wald type1;
run;
proc genmod data=a;
 model f/n=x|x|x|x|x/type1wald type1 type3wald type3;
run;
 proc glimmix data=a;
  model f/n=x|x|x|x|x/htype=1,3 e1 e3;
 run;
data p;
 x2=31.75;
 pvalue=probchi(x2,11,0);
proc print data=p;
run;
