/* Example 6.2                                     */
/* Gaussian Repeated Measures Data                 */
/* Introduces COVTEST and Fit Statistics           */
/*  to identify appropriate correlated error model */
/* Data First presented in Multivariate form       */
/*   (suitable for Proc GLM)                       */
data multivar;
 input treatment subj_id time1-time6;
 datalines;
1 1 9.326 7.936 6.175 6.891 7.468 7.018
1 2 9.086 8.013 6.247 6.462 5.355 6.236
1 3 10.584 10.348 8.622 8.429 7.252 7.825
1 4 11.866 10.975 10.391 9.143 8.201 7.861
1 5 8.624 8.301 7.016 6.012 5.753 4.328
1 6 10.703 8.907 8.624 7.54 7.262 6.9
2 1 9.471 8.05 7.073 6.65 6.942 5.949
2 2 10.356 8.661 6.789 5.682 6.026 5.879
2 3 10.379 8.068 6.443 6.247 5.798 6.56
2 4 8.062 7.143 5.179 2.281 2.888 3.633
2 5 8.843 7.51 6.779 4.497 3.918 5.723
2 6 11.198 8.301 6.854 5.331 5.532 5.756
;
proc print data=multivar;
 var treatment subj_id time1-time6;
run;
proc glm data=multivar;
 model time1-time6=treatment;
 repeated time;
 title 'old school rpt m';
run;
/* convert data to univariate form */
/* required for MIXED and GLIMMIX  */
data univar;
 set multivar;
 time=1; y=time1; output;
 time=2; y=time2; output;
 time=3; y=time3; output;
 time=4; y=time4; output;
 time=5; y=time5; output;
 time=6; y=time6; output;
proc print data=univar;
run;
/* first GLIMMIX run             */
/* unstructured covariance model */
proc glimmix data=univar;
 class treatment subject_id time;
 model y=treatment|time;
 *random intercept / subject=subject_id;
 random _residual_ / type=un subject=subject_id(treatment);
run;
/* 1st order antedependence */
proc glimmix data=univar;
 class treatment subject_id time;
 model y=treatment|time;
 *random intercept / subject=subject_id;
 random _residual_ / type=ante(1) subject=subject_id(treatment);
run;
/* 1st order autoregressive */
/* random subject effect and AR(1) covariance are identfiable here */
/* not true for UN, ANTE(1), TOEP or CS                            */
proc glimmix data=univar;
 class treatment subject_id time;
 model y=treatment|time;
 random intercept / subject=subject_id;
 random _residual_ / type=ar(1) subject=subject_id(treatment);
run;
/* AR(1) without random subject effect */
proc glimmix data=univar;
 class treatment subject_id time;
 model y=treatment|time;
 random intercept / subject=subject_id;
 *random _residual_ / type=ar(1) subject=subject_id(treatment);
run;
/* CS */
proc glimmix data=univar;
 class treatment subject_id time;
 model y=treatment|time;
 *random intercept / subject=subject_id;
 random _residual_ / type=cs subject=subject_id(treatment);
run;

