data two_way_MM_no_interaction;
 input Blk Trt Y;
datalines;
1 1 13.1
1 2 13.8
2 1 7.2
2 2 16.3
3 1 16.1
3 2 15.5
4 1 11.8
4 2 9.5
5 1 12.9
5 2 12.3
6 1 11.6
6 2 9.9
7 1 15.6
7 2 28.6
8 1 18.2
8 2 18.9
9 1 14.3
9 2 19.8
10 1 13.3
10 2 18
11 1 11
11 2 13.4
12 1 16.9
12 2 19.9
;
/* GLIMMIX run - your "answer key"  */
proc glimmix method=rspl;
 class Blk Trt;
 model Y=Trt;
 random intercept/ subject=Blk;
run;

/* now for IML  */
proc iml;
reset nolog;
use two_way_MM_no_interaction;
read all;

nobs=nrow(Y);

** set up x matrix;
xt=design(Trt);
x=j(nobs,1,1)||xt;
** cell means version: delete intercept vector => x=design(Trt);

** set up z matrix for Blk;
zb=design(Blk);
/* if you have a Blk x Trt interaction, the syntax is
zbt=hdir(xt,zb);
z=zb||zab;
*/
nb=ncol(zb);
z=zb; 
w=x || z;

** derivatives, etc we need;

zzpb=zb*zb`;
zep=i(nobs);

** give starting values of variance components;
** vector of initial values called INIT;
** INIT will be vector of previous estimates of var comp throughout iteration process;

init={1,1};    ** don't set starting value for error variance to zero;
sigma=j(2,1,.);	 ** sigma will be vector of new estimates - starts with blanks;

** maxdiff initializes covergence criterion;
** niter counts number of iterations;

maxdiff=10;
niter=0;

start;     ** start is required in IML for do-loops;

do while (maxdiff>0.00000001);

** compute current estimate of v, v-inverse, and p;
** requires you to build elements of G and R;
g=i(nb)#init[1,1];
r=i(nobs)#init[2,1];
v=z*g*z` + r;
vi=inv(v);
ri=inv(r);
gi=inv(g);

/* compute P matrix -- essential for REML estimation */
/* why? see Harville (1977)                          */ 
p=vi-vi*x*sweep(x`*vi*x)*x`*vi;


** set up mixed model equations;

rhs = x`*ri*y // z`*ri*y;

lhs = ((x`*ri*x) || (x`*ri*z)) //
      ((z`*ri*x) || (z`*ri*z + gi));
c=sweep(lhs);

** solve mixed model equations;
sol=c*rhs;

** compute score vector;
beta=sol[1:3,1];
sc1=-0.5*trace(p*zzpb)+0.5#(y-x*beta)`*vi*zzpb*vi*(y-x*beta);
sc2=-0.5*trace(p*zep)+0.5#(y-x*beta)`*vi*zep*vi*(y-x*beta);
score=sc1 // sc2;

** compute hessian matrix;
/* if you want to do Fisher Scoring, replace h11 through h33
   with elements of information matrix [E(H)]
   again, consult Harville (1977)                             */

h11=0.5*trace(p*zzpb*p*zzpb)-(y-x*beta)`*vi*zzpb*p*zzpb*vi*(y-x*beta);
h12=0.5*trace(p*zzpb*p*zep)-(y-x*beta)`*vi*zzpb*p*zep*vi*(y-x*beta);
h21=0.5*trace(p*zep*p*zzpb)-(y-x*beta)`*vi*zep*p*zzpb*vi*(y-x*beta);
h22=0.5*trace(p*zep*p*zep)-(y-x*beta)`*vi*zep*p*zep*vi*(y-x*beta);
h=(h11 || h12) // (h21 || h22);

** compute new estimate of variance components;
sigma = init - inv(h)*score;

print 'iteration' niter sigma;

/* following routine replaces negative solutions with zero  */
/* (actually -- with epsilon just greater than 0            */
/* why would you do this?                                   */
do i=1 to 2;
 if sigma[i,1]<0.00000001 then sigma[i,1]=0.00000001;
end;

reldiff=(sigma-init)/init;

if niter>0 then do;
 maxdiff=max(abs(reldiff));
end;

init=sigma;
niter=niter+1;

if niter>100 then go to noway;

end;

print 'successful convergence';

go to printit;

noway:

 print 'after 100' niter '...nothing';

goto iquit;

printit:

print niter sigma sol;

iquit:
 finish;
run;
