data two_way_MM_with_interaction;
 input Blk Trt Triple Y;
/* There are three observation per Blk x Trt combination  */
/* "Triple" identifies which of the three                 */
datalines;
1 1 1 17
1 1 2 13.7
1 1 3 15.7
1 2 1 14.1
1 2 2 19.3
1 2 3 18.6
1 3 1 13.2
1 3 2 14.4
1 3 3 10.3
2 1 1 11.4
2 1 2 11.7
2 1 3 14.4
2 2 1 20.1
2 2 2 19.9
2 2 3 20.7
2 3 1 19.4
2 3 2 17.2
2 3 3 18.1
3 1 1 9.2
3 1 2 6.7
3 1 3 9.5
3 2 1 13.1
3 2 2 14.7
3 2 3 14.7
3 3 1 20.6
3 3 2 20.2
3 3 3 18.7
4 1 1 19.4
4 1 2 23.4
4 1 3 15.2
4 2 1 19.2
4 2 2 18.3
4 2 3 15.9
4 3 1 13
4 3 2 18.9
4 3 3 14.7
5 1 1 13.5
5 1 2 10.8
5 1 3 17.5
5 2 1 20.5
5 2 2 22.1
5 2 3 20
5 3 1 16.6
5 3 2 15.7
5 3 3 17.7
6 1 1 13.9
6 1 2 16.4
6 1 3 16.7
6 2 1 16.8
6 2 2 16
6 2 3 18.5
6 3 1 20.7
6 3 2 18.6
6 3 3 16.4
7 1 1 14.5
7 1 2 17.8
7 1 3 16.8
7 2 1 15.7
7 2 2 18.6
7 2 3 17.5
7 3 1 16.4
7 3 2 17.6
7 3 3 14.6
8 1 1 12.6
8 1 2 10.8
8 1 3 11.1
8 2 1 13
8 2 2 17.9
8 2 3 20
8 3 1 16.7
8 3 2 15.8
8 3 3 16
9 1 1 11.4
9 1 2 14.4
9 1 3 6.7
9 2 1 13
9 2 2 11.6
9 2 3 8.3
9 3 1 14.2
9 3 2 10.8
9 3 3 14.8
10 1 1 17.4
10 1 2 13.6
10 1 3 20.5
10 2 1 11.9
10 2 2 12.6
10 2 3 15.2
10 3 1 12.5
10 3 2 16.9
10 3 3 15.1
;
/* your "answer key" is GLIMMIX run  */
proc glimmix data=two_way_MM_with_interaction;
 class Blk Trt;
 model Y = Trt / solution;
 random intercept Trt / subject=Blk;
run;

/* the following program solves the mixed model equations
    and computes REML variance component estimates via
    Newton-Raphson algorithm for the above data          */
proc iml;
use two_way_MM_with_interaction;
read all;

nobs=nrow(y);

** set up x matrix;
xt=design(Trt);
x=j(nobs,1,1)||xt;
print nobs x;
** cell means version:  delete intercept vector ==> x=design(Trt);

** set up z matrix;
zb=design(Blk);
ztb=hdir(xt,zb);
z=zb||ztb;
nb=ncol(zb);
ntb=ncol(ztb); 
w=x || z;

** derivatives, etc we need;

zzpb=zb*zb`;
zzptb=ztb*ztb`;
zep=i(nobs);

** give starting values of variance components;
** vector of initial values called INIT;
** INIT will be vector of previous estimates of var comp throughout iteration process;

init={1,1,1};    ** don't set variance component for error equal to zero;
sigma=j(3,1,.);	 ** sigma will be vector of new estimates - starts with blanks;

** maxdiff initializes covergence criterion;
** niter counts number of iterations;

maxdiff=10;
niter=0;

start;     ** start is required in IML for do-loops;

do while (maxdiff>0.00000001);

** compute current estimate of v, v-inverse, and p;
** requires you to build elements of G and R;
gb=i(nb)#init[1,1];
gtb=i(ntb)#init[2,1];
g=block(gb,gtb);
r=i(nobs)#init[3,1];
v=z*g*z` + r;
vi=inv(v);
ri=inv(r);
gi=inv(g);

/* compute P matrix -- essential for REML estimation */
/* why? see Harville (1977)                          */ 
p=vi-vi*x*sweep(x`*vi*x)*x`*vi;


** set up mixed model equations;

rhs = x`*ri*y // z`*ri*y;

lhs = ((x`*ri*x) || (x`*ri*z)) //
      ((z`*ri*x) || (z`*ri*z + gi));
c=sweep(lhs);

** solve mixed model equations;
sol=c*rhs;

** compute score vector;
beta=sol[1:4,1];
sc1=-0.5*trace(p*zzpb)+0.5#(y-x*beta)`*vi*zzpb*vi*(y-x*beta);
sc2=-0.5*trace(p*zzptb)+0.5#(y-x*beta)`*vi*zzptb*vi*(y-x*beta);
sc3=-0.5*trace(p*zep)+0.5#(y-x*beta)`*vi*zep*vi*(y-x*beta);
score=sc1 // sc2 // sc3;

** compute hessian matrix;
/* if you want to do Fisher Scoring, replace h11 through h33
   with elements of information matrix [E(H)]
   consult Harville (1977)                             */

h11=0.5*trace(p*zzpb*p*zzpb)-(y-x*beta)`*vi*zzpb*p*zzpb*vi*(y-x*beta);
h12=0.5*trace(p*zzpb*p*zzptb)-(y-x*beta)`*vi*zzpb*p*zzptb*vi*(y-x*beta);
h13=0.5*trace(p*zzpb*p*zep)-(y-x*beta)`*vi*zzpb*p*zep*vi*(y-x*beta);
h21=0.5*trace(p*zzptb*p*zzpb)-(y-x*beta)`*vi*zzptb*p*zzpb*vi*(y-x*beta);
h22=0.5*trace(p*zzptb*p*zzptb)-(y-x*beta)`*vi*zzptb*p*zzptb*vi*(y-x*beta);
h23=0.5*trace(p*zzptb*p*zep)-(y-x*beta)`*vi*zzptb*p*zep*vi*(y-x*beta);
h31=0.5*trace(p*zep*p*zzpb)-(y-x*beta)`*vi*zep*p*zzpb*vi*(y-x*beta);
h32=0.5*trace(p*zep*p*zzptb)-(y-x*beta)`*vi*zep*p*zzptb*vi*(y-x*beta);
h33=0.5*trace(p*zep*p*zep)-(y-x*beta)`*vi*zep*p*zep*vi*(y-x*beta);
h=(h11 || h12 || h13) // (h21 || h22 || h23) // (h31 || h32 || h33);

** compute new estimate of variance components;
sigma = init - inv(h)*score;

print 'iteration' niter sigma;

/* following routine replaces negative solutions with zero  */
/* (actually -- with epsilon just greater than 0            */
/* why would you do this?                                   */
do i=1 to 2;
 if sigma[i,1]<0.00000001 then sigma[i,1]=0.00000001;
end;

reldiff=(sigma-init)/init;

if niter>0 then do;
 maxdiff=max(abs(reldiff));
end;

init=sigma;
niter=niter+1;

if niter>5 then go to noway;

end;

print 'converged successfully in' niter 'iterations';

go to printit;

noway:

 print 'after' niter 'iterations... nothing';

goto iquit;

printit:

print niter sigma sol;

iquit:
 finish;
run;
